package ru.ldeloff.servermonitorbot.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.bots.TelegramLongPollingBot;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;
import ru.ldeloff.servermonitorbot.model.Role;
import ru.ldeloff.servermonitorbot.model.TelegramBot;
import ru.ldeloff.servermonitorbot.model.User;
import ru.ldeloff.servermonitorbot.repository.SshRepository;
import ru.ldeloff.servermonitorbot.service.TelegramBotService;
import ru.ldeloff.servermonitorbot.service.role.RoleService;
import ru.ldeloff.servermonitorbot.service.user.UserService;
import ru.ldeloff.servermonitorbot.utils.ui.TelegramBotKeyboard;
import ru.ldeloff.servermonitorbot.utils.ui.uname.UnameChatButtonAggregate;

@Service
@Slf4j
@RequiredArgsConstructor
public class TelegramBotController extends TelegramLongPollingBot {

    final TelegramBot telegramBot;
    final SshRepository sshRepository;
    final TelegramBotService telegramBotService;
    final UserService userService;
    final RoleService roleService;

    @Override
    public void onUpdateReceived(Update update) {
        Role role = getRole(update);
        if (update.hasMessage()) {
            if (update.getMessage().hasText()) {
                String messageText = update.getMessage().getText();
                switch (messageText) {
                    case "/start" -> {
                        if (role.getId() < 3) {
                            log.info("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). OK");
                            telegramBotService.firstUse(update, this);
                        } else {
                            log.warn("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                    }
                    case "Статус" -> {
                        if (role.getId() < 3) {
                            log.info("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). OK");
                            telegramBotService.getStatusSessions(update, this);
                        } else {
                            log.warn("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                    }
                    case "uname" -> {
                        if (role.getId() < 3) {
                            log.info("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). OK");
                            telegramBotService.sendUnameAggregate(update, this);
                        } else {
                            log.warn("Получена команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                    }
                    default -> {
                        if (role.getId() < 3) {
                            log.warn("Неизвестная команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). ОК");
                            telegramBotService.switchToMainMenu(update, this);
                        } else {
                            log.warn("Неизвестная команда '" + messageText
                                    + "' от " + update.getMessage().getChat().getUserName()
                                    + " (" + update.getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                    }
                }
            }
        } else if (update.hasCallbackQuery()) {
            String messageText = update.getCallbackQuery().getData();
            String [] tags = messageText.split(":");
            if (tags.length > 1) {
                switch (tags[0]) {
                    case "uname":
                        if (role.getId() < 3) {
                            log.info("Получена команда '" + messageText
                                    + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                                    + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). ОК");
                            telegramBotService.uname(update, this);
                        } else {
                            log.info("Получена команда '" + messageText
                                    + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                                    + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                        break;
                    default:
                        if (role.getId() < 3) {
                            log.warn("Неизвестная команда '" + messageText
                                    + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                                    + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). ОК");
                            telegramBotService.switchToMainMenu(update, this);
                        } else {
                            log.warn("Неизвестная команда '" + messageText
                                    + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                                    + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). Нет прав");
                            telegramBotService.notAuthorization(update, this);
                        }
                        break;
                }
            } else {
                if (role.getId() < 3) {
                    log.warn("Неверный формат команды '" + messageText
                            + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                            + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). ОК");
                    telegramBotService.switchToMainMenu(update, this);
                } else {
                    log.warn("Неверный формат команды '" + messageText
                            + "' от " + update.getCallbackQuery().getMessage().getChat().getUserName()
                            + " (" + update.getCallbackQuery().getMessage().getChat().getId() + "). Нет прав");
                    telegramBotService.notAuthorization(update, this);
                }
            }
        }
    }

    @Override
    public String getBotUsername() {
        return telegramBot.getBotUsername();
    }

    @Override
    public String getBotToken() {
        return telegramBot.getBotToken();
    }

    @Override
    public void onClosing() {
        super.onClosing();
        sshRepository.disconnectSessions();
    }

    private Role getRole(Update update) {
        Long id = null;
        if (update.hasMessage()) {
            id = update.getMessage().getChat().getId();            
        } else if (update.hasCallbackQuery()) {
            id = update.getCallbackQuery().getMessage().getChat().getId();
        }
        User user = userService.getByTelegramId(id);
        if (user == null) {
            return roleService.findRoleById(3L);
        } else {
            return user.getRole();
        }
    }
}
